<?xml version="1.0" standalone="yes"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:junos="http://xml.juniper.net/junos/*/junos"
	xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm" xmlns:ext="http://xmlsoft.org/XSLT/namespace"
	xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">

	<xsl:import href="../import/junos.xsl"/>

	<!-- Open a connection to MGD-->
	<xsl:variable name="connection" select="jcs:open()"/>

	<!-- Figure out which node is the secondary -->
	<xsl:variable name="cluster-info-rpc">
		<rpc>
			<get-route-engine-information>
				<node>local</node>
			</get-route-engine-information>
		</rpc>
	</xsl:variable>
	<xsl:variable name="cluster-info" select="jcs:execute($connection, $cluster-info-rpc)"/>
	<xsl:variable name="local-node" select="$cluster-info//multi-routing-engine-item/re-name"/>
	<xsl:variable name="remote-node">
		<xsl:choose>
			<xsl:when test="$local-node = 'node0'">node1</xsl:when>
			<xsl:when test="$local-node = 'node1'">node0</xsl:when>
		</xsl:choose>
	</xsl:variable>

	<xsl:variable name="rg0-master">
		<xsl:call-template name="get-master">
			<xsl:with-param name="redundancy-group">0</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>

	<xsl:template name="get-master">
		<xsl:param name="redundancy-group">0</xsl:param>

		<xsl:variable name="get-cluster-status">
			<rpc>
				<get-chassis-cluster-status>
					<redundancy-group>
						<xsl:value-of select="$redundancy-group"/>
					</redundancy-group>
				</get-chassis-cluster-status>
			</rpc>
		</xsl:variable>
		<xsl:variable name="cluster-status-results"
			select="jcs:execute($connection, $get-cluster-status)"/>

		<!-- Determine the master of the chassis cluster-->
		<xsl:variable name="rg-node0-priority"
			select="$cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[1]"/>
		<xsl:variable name="rg-node1-priority"
			select="$cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[2]"/>
		<xsl:choose>
			<xsl:when test="$rg-node0-priority = 'primary' and $rg-node1-priority != 'primary'">
				<!-- Node0 is the primary returning Node0-->
				<text>node0</text>
			</xsl:when>
			<xsl:when test="$rg-node0-priority != 'primary' and $rg-node1-priority= 'primary'">
				<!-- Node1 is the primary returning Node1-->
				<text>node1</text>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="copy-idp-signatures">
		<xsl:param name="from-node"/>
		<xsl:param name="to-node"/>
		<xsl:param name="path">/var/db/idpd/sec-download/</xsl:param>

		<xsl:variable name="ls-rpc">
			<rpc>
				<file-list>
					<path>
						<xsl:value-of select="$path"/>
					</path>
					<detail/>
				</file-list>
			</rpc>
		</xsl:variable>
		<xsl:variable name="files"
			select="jcs:execute($connection, $ls-rpc)//file-information[starts-with(file-permissions/@junos:format,'-')]/file-name"/>

		<!-- Loop through the files and copy every file to the new node -->
		<xsl:for-each select="$files">
			<xsl:variable name="file-copy-rpc">
				<rpc>
					<file-copy>
						<source>
							<xsl:value-of select="concat($path, '/', . )"/>
						</source>
						<destination>
							<xsl:value-of
								select="concat($remote-node, ':/var/db/idpd/sec-download/')"/>
						</destination>
					</file-copy>
				</rpc>
			</xsl:variable>

			<!--			<xsl:copy-of select="$file-copy-rpc"/>-->
			<xsl:value-of select="jcs:execute($connection, $file-copy-rpc)"/>
		</xsl:for-each>

	</xsl:template>

	<xsl:template name="install-packages">
		<xsl:param name="node"/>
		<xsl:variable name="regex-result" select="jcs:regex('node([01])', $node)"/>
		<xsl:variable name="node-id" select="$regex-result[2]"/>
		<xsl:variable name="install-package-rpc">
			<rpc>
				<command> request security idp security-package install node <xsl:value-of
						select="$node-id"/>
				</command>
			</rpc>
		</xsl:variable>

		<!--		<xsl:copy-of select="$install-package-rpc"/>-->
		<xsl:value-of select="jcs:execute($connection, $install-package-rpc)"/>

	</xsl:template>

	<xsl:template match="/">
		<op-script-results>
			<output>
				<xsl:choose>

					<xsl:when test="$local-node = $rg0-master">

						<!-- Copy the security packages to the backup node -->
						<xsl:call-template name="copy-idp-signatures">
							<xsl:with-param name="from-node" select="$local-node"/>
							<xsl:with-param name="to-node" select="$remote-node"/>
							<xsl:with-param name="path">/var/db/idpd/sec-download</xsl:with-param>
						</xsl:call-template>
						<!-- If the package was installed by nsm the files are stored in a different location -->
						<xsl:call-template name="copy-idp-signatures">
							<xsl:with-param name="from-node" select="$local-node"/>
							<xsl:with-param name="to-node" select="$remote-node"/>
							<xsl:with-param name="path">/var/db/idpd/nsm-download</xsl:with-param>
						</xsl:call-template>

						<!-- Installl the packages-->
						<xsl:call-template name="install-packages">
							<xsl:with-param name="node" select="$remote-node"/>
						</xsl:call-template>

					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="jcs:printf('Local node is not the RG0 master. Nothing to do\n')"/>
					</xsl:otherwise>
				</xsl:choose>
			</output>
		</op-script-results>
	</xsl:template>

</xsl:stylesheet>
