<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:junos="http://xml.juniper.net/junos/*/junos"
    xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm"
    xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">
    <xsl:import href="../import/junos.xsl"/>

    <!-- Template difinitions -->
    <xsl:template name="add-event-policies-rpc">
        <xsl:param name="hold-down">60</xsl:param>
        <xsl:param name="test-owner"/>
        <xsl:param name="test-name"/>
        <xsl:param name="test-duration">60</xsl:param>
        <xsl:param name="threshold"/>

        <event-options>
            
            <!-- Dampen the test completed events -->
            <policy>
                <name><xsl:value-of select="$test-owner"/>-<xsl:value-of select="$test-name"
                    />-dampen-up</name>
                <events>ping_test_completed</events>
                <within>
                    <name><xsl:value-of select="$hold-down"/></name>
                    <trigger>
                        <after/>
                        <count>1</count>
                    </trigger>
                </within>
                <attributes-match>
                    <from-event-attribute>ping_test_completed.test-name</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-name"/>
                    </to-event-attribute-value>
                </attributes-match>
                <attributes-match>
                    <from-event-attribute>ping_test_completed.test-owner</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-owner"/>
                    </to-event-attribute-value>
                </attributes-match>
                <then>
                    <ignore/>
                </then>
            </policy>

            <!-- Dampen the test failed events -->
            <policy>
                <name><xsl:value-of select="$test-owner"/>-<xsl:value-of select="$test-name"
                    />-dampen-down</name>
                <events>ping_test_failed</events>
                <within>
                    <name><xsl:value-of select="$hold-down"/></name>
                    <trigger>
                        <after/>
                        <count>1</count>
                    </trigger>
                </within>
                <attributes-match>
                    <from-event-attribute>ping_test_failed.test-name</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-name"/>
                    </to-event-attribute-value>
                </attributes-match>
                <attributes-match>
                    <from-event-attribute>ping_test_failed.test-owner</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-owner"/>
                    </to-event-attribute-value>
                </attributes-match>
                <then>
                    <ignore/>
                </then>
            </policy>

            <!-- Dampen threshold events (if configured) -->
            <xsl:if test="$threshold != 'false'">
                <policy>
                    <name><xsl:value-of select="$test-owner"/>-<xsl:value-of select="$test-name"
                        />-dampen-threshold</name>
                    <events>
                        <xsl:value-of select="$threshold"/>
                    </events>
                    <within>
                        <name>
                            <xsl:value-of select="$hold-down"/>
                        </name>
                        <events>
                            <xsl:value-of select="$threshold"/>
                        </events>
                        <trigger>
                            <after/>
                            <count>4</count>
                        </trigger>
                    </within>
                    <attributes-match>
                        <from-event-attribute><xsl:value-of select="$threshold"
                            />.test-name</from-event-attribute>
                        <condition>matches</condition>
                        <to-event-attribute-value>
                            <xsl:value-of select="$test-name"/>
                        </to-event-attribute-value>
                    </attributes-match>
                    <attributes-match>
                        <from-event-attribute><xsl:value-of select="$threshold"
                            />.test-owner</from-event-attribute>
                        <condition>matches</condition>
                        <to-event-attribute-value>
                            <xsl:value-of select="$test-owner"/>
                        </to-event-attribute-value>
                    </attributes-match>
                    <then>
                        <ignore/>
                    </then>
                </policy>
            </xsl:if>

            <!-- Trigger an up event if we receive a test_completed and no threshold errors -->
            <policy>
                <name><xsl:value-of select="$test-owner"/>-<xsl:value-of select="$test-name"
                    />-up</name>
                <events>ping_test_completed</events>
                <xsl:if test="$threshold != 'false'">
                    <within>
                        <name>
                            <xsl:value-of select="$test-duration"/>
                        </name>
                        <not>
                            <events>
                                <xsl:value-of select="$threshold"/>
                            </events>
                        </not>
                    </within>
                </xsl:if>
                <attributes-match>
                    <from-event-attribute>ping_test_completed.test-name</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-name"/>
                    </to-event-attribute-value>
                </attributes-match>
                <attributes-match>
                    <from-event-attribute>ping_test_completed.test-owner</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-owner"/>
                    </to-event-attribute-value>
                </attributes-match>
                <xsl:if test="$threshold != 'false'">
                    <attributes-match>
                        <from-event-attribute><xsl:value-of select="$threshold"
                            />.test-name</from-event-attribute>
                        <condition>matches</condition>
                        <to-event-attribute-value>
                            <xsl:value-of select="$test-name"/>
                        </to-event-attribute-value>
                    </attributes-match>
                    <attributes-match>
                        <from-event-attribute><xsl:value-of select="$threshold"
                            />.test-owner</from-event-attribute>
                        <condition>matches</condition>
                        <to-event-attribute-value>
                            <xsl:value-of select="$test-owner"/>
                        </to-event-attribute-value>
                    </attributes-match>
                </xsl:if>
                <then>
                    <event-script>
                        <name>rpm-monitor.xslt</name>
                        <arguments>
                            <name>event-name</name>
                            <value>ping_test_up</value>
                        </arguments>
                        <arguments>
                            <name>test-owner</name>
                            <value>{$$.test-owner}</value>
                        </arguments>
                        <arguments>
                            <name>test-name</name>
                            <value>{$$.test-name}</value>
                        </arguments>
                    </event-script>
                </then>
            </policy>

            <!-- Trigger a down event if we receive a test_failed -->
            <policy>
                <name><xsl:value-of select="$test-owner"/>-<xsl:value-of select="$test-name"
                    />-down</name>
                <events>ping_test_failed</events>
                <attributes-match>
                    <from-event-attribute>ping_test_failed.test-name</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-name"/>
                    </to-event-attribute-value>
                </attributes-match>
                <attributes-match>
                    <from-event-attribute>ping_test_failed.test-owner</from-event-attribute>
                    <condition>matches</condition>
                    <to-event-attribute-value>
                        <xsl:value-of select="$test-owner"/>
                    </to-event-attribute-value>
                </attributes-match>
                <then>
                    <event-script>
                        <name>rpm-monitor.xslt</name>
                        <arguments>
                            <name>test-owner</name>
                            <value>{$$.test-owner}</value>
                        </arguments>
                        <arguments>
                            <name>test-name</name>
                            <value>{$$.test-name}</value>
                        </arguments>
                        <arguments>
                            <name>event-name</name>
                            <value>ping_test_updown</value>
                        </arguments>
                    </event-script>
                </then>
            </policy>

            <!-- Trigger a down event if we receive 3 consecutive treshold alarms -->
            <xsl:if test="$threshold != 'false' ">
                <policy>
                    <name><xsl:value-of select="$test-owner"/>-<xsl:value-of select="$test-name"
                        />-down-threshold</name>
                    <events>
                        <xsl:value-of select="$threshold"/>
                    </events>
                    <within>
                        <name>
                            <xsl:value-of select="$hold-down"/>
                        </name>
                        <events>
                            <xsl:value-of select="$threshold"/>
                        </events>
                        <trigger>
                            <on/>
                            <count>3</count>
                        </trigger>
                    </within>
                    <attributes-match>
                        <from-event-attribute><xsl:value-of select="$threshold"
                            />.test-name</from-event-attribute>
                        <condition>matches</condition>
                        <to-event-attribute-value>
                            <xsl:value-of select="$test-name"/>
                        </to-event-attribute-value>
                    </attributes-match>
                    <attributes-match>
                        <from-event-attribute><xsl:value-of select="$threshold"
                            />.test-owner</from-event-attribute>
                        <condition>matches</condition>
                        <to-event-attribute-value>
                            <xsl:value-of select="$test-owner"/>
                        </to-event-attribute-value>
                    </attributes-match>
                    <then>
                        <event-script>
                            <name>rpm-monitor.xslt</name>
                            <arguments>
                                <name>test-owner</name>
                                <value>{$$.test-owner}</value>
                            </arguments>
                            <arguments>
                                <name>test-name</name>
                                <value>{$$.test-name}</value>
                            </arguments>
                            <arguments>
                                <name>event-name</name>
                                <value>ping_threshold_down</value>
                            </arguments>
                        </event-script>
                    </then>
                </policy>
            </xsl:if>
        </event-options>

    </xsl:template>

    <!--
        Checking the macro config
    -->
    <xsl:template
        match="//routing-options/static/route//apply-macro[starts-with(name,'rpm-monitor')]">

        <xsl:variable name="hold-down" select="data[name='hold-down']/value"/>
        <xsl:variable name="test-name" select="data[name='test-name']/value"/>
        <xsl:variable name="test-owner" select="data[name='test-owner']/value"/>
        <xsl:variable name="weight" select="data[name='weight']/value"/>

        <xsl:if test="not($test-name)">
            <xnm:error>
                <xsl:call-template name="jcs:edit-path"/>
                <message>missing mandatory parameter test-name</message>
            </xnm:error>
        </xsl:if>

        <xsl:if test="not($test-owner)">
            <xnm:error>
                <xsl:call-template name="jcs:edit-path"/>
                <message>missing mandatory parameter test-owner</message>
            </xnm:error>
        </xsl:if>

        <xsl:if test="not($weight)">
            <xnm:error>
                <xsl:call-template name="jcs:edit-path"/>
                <message>missing mandatory parameter weight</message>
            </xnm:error>
        </xsl:if>

        <!-- Check the weight -->
        <xsl:if test="number($weight) &lt; 0 or number($weight) &gt; 65535">
            <xnm:error>
                <xsl:call-template name="jcs:edit-path"/>
                <message>weight must be a number between (1...65535)</message>
            </xnm:error>
        </xsl:if>

        <!-- Check the hold-down -->
        <xsl:if test="number($hold-down) &lt; 60 or number($hold-down) &gt; 8640">
            <xnm:error>
                <xsl:call-template name="jcs:edit-path"/>
                <message>hold-down must be a number between (60...8640)</message>
            </xnm:error>
        </xsl:if>

        <!-- Check the existance of a probe -->
        <xsl:if test="not(//services/rpm/probe[name=$test-owner]/test[name=$test-name])">
            <xnm:error>
                <xsl:call-template name="jcs:edit-path"/>
                <message>rpm owner <xsl:value-of select="$test-owner"/> test <xsl:value-of
                        select="$test-name"/> does not exist</message>
            </xnm:error>
        </xsl:if>

    </xsl:template>

    <xsl:template match="configuration">
        <xsl:apply-templates/>
        
        <!-- The macro config is fine, now let's add the required config -->
        <xsl:for-each select="//services/rpm/probe">
            <xsl:variable name="test-owner" select="name"/>
            <xsl:for-each select="./test">
                <xsl:variable name="test-name" select="name"/>

                <xsl:if
                    test="//routing-options/static/route//apply-macro[starts-with(name,'rpm-monitor')]/data[
                  name='test-name' and value=$test-name 
                  and ../data[name='test-owner' and value=$test-owner]] or
                  //interfaces/interface/apply-macro[starts-with(name,'rpm-monitor')]/data[
                  name='test-name' and value=$test-name 
                  and ../data[name='test-owner' and value=$test-owner]] or
				//interfaces/interface/unit/apply-macro[starts-with(name,'rpm-monitor')]/data[
                name='test-name' and value=$test-name
                and ../data[name='test-owner' and value=$test-owner]]

                  ">
                    <xsl:variable name="test-duration" select="(probe-count * probe-interval) + test-interval"/>
                    <xsl:variable name="threshold">
                        <xsl:choose>
                            <xsl:when
                                test="thresholds/rtt">
                                <xsl:text>ping_rtt_threshold_exceeded</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/jitter-rtt">
                                <xsl:text>ping_rtt_jtr_thresh_exceed</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/std-dev-rtt">
                                <xsl:text>ping_rtt_stddv_thresh_exceed</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/egress-time">
                                <xsl:text>ping_egress_threshold_exceeded</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/ingress-time">
                                <xsl:text>ping_ingress_threshold_exceeded</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/jitter-egress">
                                <xsl:text>ping_egress_jitter_thresh_exceed</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/jitter-ingress">
                                <xsl:text>ping_ingress_jtr_thresh_exceed</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/std-dev-egress">
                                <xsl:text>ping_egress_stddev_thresh_exceed</xsl:text>
                            </xsl:when>
                            <xsl:when
                                test="thresholds/std-dev-ingress">
                                <xsl:text>ping_ingress_stddv_thresh_exceed</xsl:text>
                            </xsl:when>
                            <xsl:otherwise>false</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>

                    <transient-change>
                        <xsl:call-template name="add-event-policies-rpc">
                            <xsl:with-param name="test-name" select="$test-name"/>
                            <xsl:with-param name="test-owner" select="$test-owner"/>
                            <xsl:with-param name="threshold" select="$threshold"/>
                        </xsl:call-template>
                    </transient-change>
                </xsl:if>
            </xsl:for-each>
        </xsl:for-each>

    </xsl:template>


</xsl:stylesheet>
