<?xml version="1.0" standalone="yes"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:junos="http://xml.juniper.net/junos/*/junos"
	xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm" 
	xmlns:ext="http://xmlsoft.org/XSLT/namespace"
	xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">
	
	<xsl:import href="../import/junos.xsl"/>

	<xsl:variable name="arguments">
		<argument>
			<name>aggregate</name>
			<description>Aggregation criteria (source-address,destination-address,etc)</description>
		</argument>
		<argument>
			<name>filename</name>
			<description>Name of the log file</description>
		</argument>
	</xsl:variable>
	<xsl:param name="aggregate">source-address</xsl:param>
	<xsl:param name="filename">default-log-messages</xsl:param>
	

	<!-- Accept some shortrands for the aggregate parameter
		for instance, source-zone is the same as source-zone-name -->	
	<xsl:variable name="aggregate-by">
		<xsl:choose>
			<xsl:when test="$aggregate = 'source'">source-address</xsl:when>
			<xsl:when test="$aggregate = 'destination'">destination-address</xsl:when>
			<xsl:when test="$aggregate = 'source-zone' or $aggregate = 'source'">source-zone-name</xsl:when>
			<xsl:when test="$aggregate = 'destination-zone'">destination-zone-name</xsl:when>
			<xsl:when test="$aggregate = 'service'">service-name</xsl:when>
			<xsl:when test="$aggregate = 'policy'">policy-name</xsl:when>
			<xsl:when test="$aggregate = 'protocol'">protocol-id</xsl:when>
			<xsl:when test="$aggregate = 'port'">destination-port</xsl:when>
			<xsl:otherwise><xsl:value-of select="$aggregate"/></xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	

	<!-- 
		session-logs:: Global variable that stores the firewall log
	-->
	<xsl:variable name="get-session-log-rpc">
		<rpc>
			<command>show log <xsl:value-of select="$filename"/></command>
		</rpc>
	</xsl:variable>
	<xsl:variable name="session-logs" select="jcs:break-lines(jcs:invoke($get-session-log-rpc))"/>

	<!-- 
		Parse the session logs and store them in a variable
		This can use up a bunch of memory, but I haven't looked into reducing the mem footprint
	-->
	<xsl:variable name="sessions">
		<xsl:for-each select="$session-logs">
			<!-- Skip empty lines -->
			<xsl:if test="string(.)">
				<xsl:variable name="structured-part"
					select="jcs:regex('RT_FLOW_SESSION_CLOSE \[junos@2636.1.1.1.2.39 (.+)&quot;\]',.)"/>

				<xsl:if test="$structured-part[2]">

					<xsl:variable name="avps"
					select="jcs:split('(=&quot;|&quot; )', $structured-part[2])"/>
					
					<session>
						<xsl:for-each select="$avps[(position() mod 2) = 1]">
							<xsl:element name="{.}">
								<xsl:value-of select="following::*"/>
							</xsl:element>
						</xsl:for-each>
					</session>
				</xsl:if>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	
	<xsl:variable name="session-set" select="ext:node-set($sessions)"/>

	<xsl:template match="/">
	
		<op-script-results>
			<output>
				<xsl:value-of select="jcs:printf('%-20s %-20s %-20s %-20s &#10;',$aggregate-by,'Packets', 'Bytes', 'Sessions')"/>
				
				<xsl:for-each select="$session-set/session/*[name() = $aggregate-by and 
									not( text() = ../preceding-sibling::session/*[name() = $aggregate-by]/text() ) ]/text()">
							<xsl:variable name="record" select="."/>
							<xsl:value-of select="jcs:printf('%-20s %-20d %-20s %-20s &#10;',$record,
								sum($session-set/session/*[name() = $aggregate-by and text() = $record]/../packets-from-client) + sum($session-set/session/*[name() = $aggregate-by and text() = $record]/../packets-from-server),
								sum($session-set/session/*[name() = $aggregate-by and text() = $record]/../bytes-from-client) + sum($session-set/session/*[name() = $aggregate-by and text() = $record]/../bytes-from-server),
								count($session-set/session/*[name() = $aggregate-by and text() = $record])
								)"/>
						</xsl:for-each>			
			</output>
		</op-script-results>

	</xsl:template>

</xsl:stylesheet>
