<?xml version="1.0" standalone="yes"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:junos="http://xml.juniper.net/junos/*/junos"
	xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm" 
	xmlns:ext="http://xmlsoft.org/XSLT/namespace"
	xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">
	<xsl:import href="../import/junos.xsl"/>

	<xsl:template match="configuration/security">

		<xsl:variable name="zone" select="./zones/security-zone[apply-macro[name='zone-group']]"/>

		<!--		 First check that they don't use a global policy both as source AND destination -->
		<!--	<xsl:if test="//policies/policy[from-zone-name=$zone/name and to-zone-name=$zone/name]">
			<xnm:error>
				<message>Zone context cannot use two global zones!</message>
			</xnm:error>
		</xsl:if>-->

		<!-- Apply the address book entries to each contituent zone -->
		<xsl:for-each select="$zone[address-book]">
			<!--			<xnm:warning><message>Address book entry found in global zone</message></xnm:warning>-->
			<xsl:for-each select="./apply-macro[name='zone-group']/data/name">
				<!--<xnm:warning>
					<message>-->
				<transient-change>
					<security>
						<zones>
							<security-zone>
								<name>
									<xsl:value-of select="."/>
								</name>
								<xsl:copy-of select="../../../address-book"/>
							</security-zone>
						</zones>
					</security>
				</transient-change>
				<!--</message>
					</xnm:warning>-->
			</xsl:for-each>
		</xsl:for-each>

		<xsl:variable name="policies">

			<xsl:for-each select="//policies/policy[from-zone-name=$zone/name]">
				<xsl:variable name="policy" select="./policy"/>
				<xsl:variable name="to-zone" select="./to-zone-name"/>
				<xsl:variable name="from-zone" select="./from-zone-name"/>

				<xsl:choose>
					<!-- Both source and dst zones are group zones -->
					<xsl:when test="$zone[name=$to-zone]">
						
						
						<xsl:for-each
							select="$zone[name=$from-zone]/apply-macro[name='zone-group']/data/name">
							<xsl:variable name="source-zone" select="."/>
							<xsl:for-each
								select="$zone[name=$to-zone]/apply-macro[name='zone-group']/data/name">
								<xsl:variable name="destination-zone" select="."/>
								<policy>
									<from-zone-name>
										<xsl:value-of select="$source-zone"/>
									</from-zone-name>
									<to-zone-name>
										<xsl:value-of select="$destination-zone"/>
									</to-zone-name>
									<xsl:copy-of select="$policy"/>
								</policy>
							</xsl:for-each>
						</xsl:for-each>
					</xsl:when>

					<!-- Only the source zone is a group zone -->
					<xsl:otherwise>
						<xsl:for-each
							select="$zone[name=$from-zone]/apply-macro[name='zone-group']/data/name">
							<policy>
								<from-zone-name>
									<xsl:value-of select="."/>
								</from-zone-name>
								<to-zone-name>
									<xsl:value-of select="$to-zone"/>
								</to-zone-name>
								<xsl:copy-of select="$policy"/>
							</policy>
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>

			</xsl:for-each>

			<xsl:for-each select="//policies/policy[to-zone-name=$zone/name]">
				<xsl:variable name="policy" select="./policy"/>
				<xsl:variable name="to-zone" select="./to-zone-name"/>
				<xsl:variable name="from-zone" select="./from-zone-name"/>

				<!-- Expand only if the source zone is not a group zone -->
				<xsl:if test="not($zone[name=$from-zone])">
					<xsl:for-each
						select="$zone[name=$to-zone]/apply-macro[name='zone-group']/data/name">
						<policy>
							<from-zone-name>
								<xsl:value-of select="$from-zone"/>
							</from-zone-name>
							<to-zone-name>
								<xsl:value-of select="."/>
							</to-zone-name>
							<xsl:copy-of select="$policy"/>
						</policy>
					</xsl:for-each>
				</xsl:if>
			</xsl:for-each>

		</xsl:variable>

		<!--
		<xnm:warning>
			<message>-->
		<transient-change>
			<security>
				<policies>
					<xsl:copy-of select="$policies"/>
				</policies>
			</security>
		</transient-change>
		<!--</message>
		</xnm:warning>
-->
	</xsl:template>

</xsl:stylesheet>
