<?xml version="1.0" standalone="yes"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:junos="http://xml.juniper.net/junos/*/junos"
	xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm"
	xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">

	<xsl:import href="../import/junos.xsl"/>

	<xsl:variable name="arguments">
		<argument>
			<name>zone</name>
			<description>Show interfaces from specified zone</description>
		</argument>
	</xsl:variable>
	<xsl:param name="name"/>

	<!-- Open a persistent connection -->
	<xsl:variable name="connection" select="jcs:open()"/>

	<!-- 
		tables:: Global variable that stores the routing tables info of the system
	-->
	<xsl:variable name="tables-rpc">
		<rpc>
			<command>
				show route instance detail
			</command>
		</rpc>
	</xsl:variable>
	<xsl:variable name="tables" select="jcs:execute($connection, $tables-rpc)"/>

	<xsl:template name="printzone">

		<!--Determine the status of the interface-->
		<xsl:variable name="status">
			<xsl:choose>
				<xsl:when test="if-config-flags/iff-down">
					<xsl:text>Admin Down</xsl:text>
				</xsl:when>
				<xsl:when test="if-config-flags/iff-device-down or if-config-flags/iff-link-down">
					<xsl:text>Down</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>Up</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="description">
			<xsl:choose>
				<xsl:when test="description">
					<xsl:value-of select="description"/>
				</xsl:when>
				<xsl:when test="../description">
					<xsl:value-of select="concat(../description, ' (parent)')"/>
				</xsl:when>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="intf-name" select="name"/>

		<xsl:variable name="routing-instance">
			<xsl:choose>
				<xsl:when test="$tables//instance-core[instance-interface/interface-name=$intf-name]/instance-name">
					<xsl:value-of select="$tables//instance-core[instance-interface/interface-name=$intf-name]/instance-name" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>Master</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

<!--		<xsl:copy-of select="$tables//instance-core[instance-interface/interface-name=$intf-name]"/>-->

		<!--Print ifd name, zone, and status-->
		<xsl:value-of
			select="jcs:printf('%-20jls %-15s %-15s %-25s %-10s %s &#10;',
			name, address-family/interface-address/ifa-local, logical-interface-zone-name, 
			$routing-instance, $status, $description)"/>

	</xsl:template>

	<xsl:template match="/">
		<op-script-results>
			<xsl:variable name="rpc">
				<rpc>
					<get-interface-information junos:style="detail"/>
				</rpc>
			</xsl:variable>
			<xsl:variable name="interfaces" select="jcs:execute($connection, $rpc)"/>

			<output>

				<xsl:value-of
					select="jcs:printf('%-20jls %-15s %-15s %-25s %-10s %s &#10;', 'Logical Interface', 'Address', 'Zone', 'Routing Instance','Status', 'Description')"/>

				<xsl:choose>
					<xsl:when test="$name">
						<xsl:for-each
							select="$interfaces/physical-interface/logical-interface[address-family/address-family-name='inet'
																									and logical-interface-zone-name=$name]">
							<xsl:call-template name="printzone"/>
						</xsl:for-each>
					</xsl:when>

					<xsl:otherwise>
						<xsl:for-each
							select="$interfaces/physical-interface[not(starts-with(name,'em')) and not(starts-with(name,'fxp')) and not(starts-with(name,'pc-')) 
							and not(name='.local.') and not(starts-with(name,'fab'))]
								/logical-interface[address-family/address-family-name='inet' and not(starts-with(name,'lo0.1638'))]">
							<xsl:call-template name="printzone"/>
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>

			</output>

		</op-script-results>
	</xsl:template>

</xsl:stylesheet>
