<?xml version="1.0" standalone="yes"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:junos="http://xml.juniper.net/junos/*/junos"
	xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm"
	xmlns:ext="http://xmlsoft.org/XSLT/namespace"
	xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">

	<xsl:import href="../import/junos.xsl"/>

	<!-- Open a persistent connection -->
	<xsl:variable name="connection" select="jcs:open()"/>

	<xsl:variable name="format-string" select="'%-12s %-12s %-15j1s %-15s %-15s %-12s %-6s %j1s &#10;'"/>

	<xsl:param name="expand" select="false()"/>


	<xsl:template name="print-policy">
		<xsl:param name="name"/>
		<xsl:param name="from-zone"/>
		<xsl:param name="to-zone"/>
		<xsl:param name="source-address"/>
		<xsl:param name="destination-address"/>
		<xsl:param name="application"/>
		<xsl:param name="action"/>
		<xsl:param name="comment"/>
		<xsl:param name="row" select="number('1')"/>
		
		<xsl:variable name="num-rows">
			<xsl:choose>
				<xsl:when
					test="count($application) &gt;= count($source-address) and count($application) &gt;= count($destination-address)">
					<xsl:value-of select="count($application)"/>
				</xsl:when>
				<xsl:when
					test="count($source-address) &gt; count($application) and count($source-address) &gt; count($destination-address)">
					<xsl:value-of select="count($source-address)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count($destination-address)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:if test="$row &lt;= $num-rows">
			<xsl:value-of
				select="jcs:printf($format-string,
					$from-zone,
					$to-zone,
					$name,
					$source-address[$row],
					$destination-address[$row],
					$application[$row],
					$action,
					$comment
					)"/>
			<xsl:call-template name="print-policy">
				<xsl:with-param name="row" select="$row+1"/>
				<xsl:with-param name="num-rows" select="$num-rows"/>
				<xsl:with-param name="source-address" select="$source-address"/>
				<xsl:with-param name="destination-address" select="$destination-address"/>
				<xsl:with-param name="application" select="$application"/>
			</xsl:call-template>

		</xsl:if>

	</xsl:template>

	<xsl:template match="/">
		<op-script-results>
			<output>

				<xsl:choose>
					<xsl:when test="$expand">
						<!-- Get the info from the data plane -->
						<xsl:variable name="rpc">
							<command>show security policies</command>
						</xsl:variable>
						<xsl:variable name="policies" select="jcs:execute($connection, $rpc)"/>

						<xsl:value-of
							select="jcs:printf($format-string,
							'From-Zone', 'To-Zone', 'Name', 'Src-Addr', 'Dst-Addr', 'Application', 'Action','')"/>

						<!-- Loop through each context -->
						<xsl:for-each select="$policies//security-context">

							<xsl:variable name="from-zone"
								select="context-information/source-zone-name"/>
							<xsl:variable name="to-zone"
								select="context-information/destination-zone-name"/>

							<xsl:for-each select="policies//policy-information">

								<xsl:call-template name="print-policy">
									<xsl:with-param name="from-zone"
										select="$from-zone"/>
									<xsl:with-param name="to-zone"
										select="$to-zone"/>
									<xsl:with-param name="name" select="policy-name"/>
									<xsl:with-param name="destination-address"
										select="destination-addresses/destination-address/address-name"/>
									<xsl:with-param name="source-address" select="source-addresses/source-address/address-name"/>
									<xsl:with-param name="application" select="applications/application/application-name"/>
									<xsl:with-param name="action" select=".//action-type"/>

								</xsl:call-template>
							</xsl:for-each>

						</xsl:for-each>


					</xsl:when>


					<xsl:otherwise>
						<!-- Get the info from the config database -->
						<xsl:variable name="rpc">
							<get-configuration database="committed">
								<configuration>
									<security>
										<policies>
											<policy/>
										</policies>
									</security>
								</configuration>
							</get-configuration>
						</xsl:variable>
						<xsl:variable name="config" select="jcs:execute($connection, $rpc)"/>
						<xsl:variable name="policies" select="$config//policies"/>

						<xsl:value-of
							select="jcs:printf($format-string,
							'From-Zone', 'To-Zone', 'Name', 'Src-Addr', 'Dst-Addr', 'Application', 'Action', 'Comment')"/>

						<!-- Loop through each context -->
						<xsl:for-each select="$policies/policy/policy">
							<xsl:variable name="name" select="name"/>

							<xsl:call-template name="print-policy">
								<xsl:with-param name="from-zone" select="../from-zone-name"/>
								<xsl:with-param name="to-zone" select="../to-zone-name"/>
								<xsl:with-param name="name" select="name"/>
								<xsl:with-param name="destination-address"
									select="match/destination-address"/>
								<xsl:with-param name="source-address" select="match/source-address"/>
								<xsl:with-param name="application" select="match/application"/>
								<xsl:with-param name="action">
									<xsl:choose>
										<xsl:when test="./then/permit">
											<xsl:text>permit</xsl:text>
										</xsl:when>
										<xsl:when test="./then/deny">
											<xsl:text>deny</xsl:text>
										</xsl:when>
									</xsl:choose>
								</xsl:with-param>
								<xsl:with-param name="comment"
									select="$policies/policy/comment[following-sibling::policy[1]/name=$name]"
								/>
							</xsl:call-template>

						</xsl:for-each>


					</xsl:otherwise>
				</xsl:choose>

			</output>
		</op-script-results>
	</xsl:template>

</xsl:stylesheet>
