<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:junos="http://xml.juniper.net/junos/*/junos"
    xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm" xmlns:ext="http://xmlsoft.org/XSLT/namespace"
    xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">
    <xsl:import href="../import/junos.xsl"/>

    <!-- rpm-monitor.xslt:
            This script is used as an event script to trigger a failover in a 
            route metric based on the result of a probe.
            The script receives the probe name owner and event-name. 
            Based on the test results it goes through the configuration and changes the objects being tracked
            by the probe.
    -->

    <!-- Argument declaration -->
    <xsl:variable name="arguments">
        <argument>
            <name>test-owner</name>
            <description>rpm test owner</description>
        </argument>
        <argument>
            <name>test-name</name>
            <description>rpm test name</description>
        </argument>
        <argument>
            <name>event-name</name>
            <description>Event that triggered the script</description>
        </argument>
    </xsl:variable>
    <xsl:param name="test-owner"/>
    <xsl:param name="test-name"/>
    <xsl:param name="event-name"/>

    <!-- Template Definitions -->
    <!-- Open a persistent connection -->
    <xsl:variable name="connection" select="jcs:open()"/>

    <!-- $configuration: Stores the relevant sections of the config -->
    <xsl:variable name="get-config-rpc">
        <rpc>
            <get-configuration>
                <configuration>

                    <routing-options>
                        <static>
                            <route/>
                        </static>
                    </routing-options>

                    <interfaces>
                        <interface/>
                    </interfaces>



                </configuration>
            </get-configuration>

        </rpc>
    </xsl:variable>
    <xsl:variable name="configuration" select="jcs:execute($connection, $get-config-rpc)"/>

    <!-- $routes: Stores all the valid routes/ifds/ifls tracking the probe -->
    <xsl:variable name="routes"
        select="$configuration//route[
        .//apply-macro[starts-with(name,'rpm-monitor') and data[name='test-owner' and value=$test-owner] 
        and data[name='test-name' and value=$test-name]
        and data[name='weight']
        ] ]"/>
    <xsl:variable name="ifds"
        select="$configuration//interface[
        apply-macro[starts-with(name,'rpm-monitor') and data[name='test-owner' and value=$test-owner] 
        and data[name='test-name' and value=$test-name]
        ] ]"/>
    <xsl:variable name="ifls"
        select="$configuration//interface/unit[
        apply-macro[starts-with(name,'rpm-monitor') and data[name='test-owner' and value=$test-owner] 
        and data[name='test-name' and value=$test-name]
        ] ]"/>

    <!--     and data[name='weight']-->

    <!-- get-current-time:
            Determine current time -->
    <xsl:template name="get-current-time">
        <xsl:variable name="rpc-get-current-time">
            <rpc>
                <get-system-uptime-information/>
            </rpc>
        </xsl:variable>
        <xsl:variable name="results-get-current-time"
            select="jcs:execute($connection, $rpc-get-current-time)"/>
        <xsl:choose>
            <xsl:when test="$results-get-current-time/multi-routing-engine-item">
                <xsl:variable name="current-time"
                    select="$results-get-current-time/multi-routing-engine-item[1]/system-uptime-information/current-time/date-time/@junos:seconds"/>
                <text>
                    <xsl:value-of select="$current-time"/>
                </text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="current-time"
                    select="$results-get-current-time/current-time/date-time/@junos:seconds"/>
                <text>
                    <xsl:value-of select="$current-time"/>
                </text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- metric-down:
            Modifies the route metric, based on the configured weight for the rpm test -->
    <xsl:template name="metric-down">
        <xsl:param name="test-owner"/>
        <xsl:param name="test-name"/>

        <!-- First, check if we have anything to change
             Make sure none of the routes has a hold-down-time configured
        -->

        <xsl:choose>
            <xsl:when
                test="count(
                    $routes//apply-macro[
                        data[name='test-name' and value=$test-name] and 
                        data[name='test-owner' and value=$test-owner] and
                        not(data[name = 'hold-down-time']) ])">

                <!-- Get current time -->
                <xsl:variable name="current-time">
                    <xsl:call-template name="get-current-time"/>
                </xsl:variable>

                <!-- Open a private config -->
                <xsl:variable name="rpc-configure-private">
                    <rpc>
                        <open-configuration>
                            <private/>
                        </open-configuration>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>

                <!-- Update all the routes with unqualified next-hops -->
                <xsl:for-each
                    select="$routes/apply-macro[
                        data[name='test-name' and value=$test-name] and 
                        data[name='test-owner' and value=$test-owner] and
                        not(data[name = 'hold-down-time']) ]">

                    <!-- Extract some info from the macro and the next-hop's metric -->
                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="weight" select="./data[name='weight']/value"/>
                    <xsl:variable name="hold-down-time">
                        <xsl:choose>
                            <xsl:when test="./data[name='hold-down']/value">
                                <xsl:value-of
                                    select="./data[name='hold-down']/value + $current-time"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="300 +  $current-time"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="metric">
                        <xsl:choose>
                            <xsl:when test="../metric">
                                <xsl:value-of select="../metric + $weight"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$weight + 1"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="route-name" select="../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <routing-options>
                                        <static>
                                            <route>
                                                <name>
                                                  <xsl:value-of select="$route-name"/>
                                                </name>
                                                <metric>
                                                  <xsl:value-of select="$metric"/>
                                                </metric>
                                                <apply-macro>
                                                  <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                  </name>
                                                  <data>
                                                  <name>hold-down-time</name>
                                                  <value>
                                                  <xsl:value-of select="$hold-down-time"/>
                                                  </value>
                                                  </data>
                                                </apply-macro>
                                            </route>
                                        </static>
                                    </routing-options>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('Metric changed for route ',
                                        $route-name,
                                        ' to ',
                                        $metric, 
                                        ' due to a down event'))"/>

                </xsl:for-each>

                <!-- Update all the routes with qualified next-hops -->
                <xsl:for-each
                    select="$routes/qualified-next-hop/apply-macro[
                    data[name='test-name' and value=$test-name] and 
                    data[name='test-owner' and value=$test-owner] and
                    not(data[name = 'hold-down-time']) ]">

                    <!-- Extract some info from the macro and the next-hop's metric -->
                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="weight" select="./data[name='weight']/value"/>
                    <xsl:variable name="hold-down-time">
                        <xsl:choose>
                            <xsl:when test="./data[name='hold-down']/value">
                                <xsl:value-of
                                    select="./data[name='hold-down']/value + $current-time"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="300 +  $current-time"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="metric">
                        <xsl:choose>
                            <xsl:when test="../metric">
                                <xsl:value-of select="../metric + $weight"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$weight + 1"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="next-hop-name" select="../name"/>
                    <xsl:variable name="route-name" select="../../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <routing-options>
                                        <static>
                                            <route>
                                                <name>
                                                  <xsl:value-of select="$route-name"/>
                                                </name>
                                                <qualified-next-hop>
                                                  <name>
                                                  <xsl:value-of select="$next-hop-name"/>
                                                  </name>
                                                  <metric>
                                                  <xsl:value-of select="$metric"/>
                                                  </metric>
                                                  <apply-macro>
                                                  <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                  </name>
                                                  <data>
                                                  <name>hold-down-time</name>
                                                  <value>
                                                  <xsl:value-of
                                                  select="$hold-down-time"/>
                                                  </value>
                                                  </data>
                                                  </apply-macro>
                                                </qualified-next-hop>
                                            </route>
                                        </static>
                                    </routing-options>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('Metric changed for route ',
                        $route-name,  ' next-hop ', $next-hop-name, ' to ', $metric, ' due to a down event'))"/>

                </xsl:for-each>

                <!-- Commit the config changes -->
                <xsl:variable name="commit-config-rpc">
                    <rpc>
                        <commit-configuration/>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $commit-config-rpc)"/>

            </xsl:when>

            <xsl:otherwise>
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe down event received, but there is nothing to do with the routes')"
                />
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>

    <xsl:template name="ifd-up">
        <xsl:param name="test-owner"/>
        <xsl:param name="test-name"/>

        <!-- First, check if we have anything to change
            Make sure none of the interfaces has a hold-down-time configured
        -->

        <xsl:choose>
            <xsl:when
                test="count(
                $ifds//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                not(data[name = 'hold-down-time'])])">



                <!-- Get current time -->
                <xsl:variable name="current-time">
                    <xsl:call-template name="get-current-time"/>
                </xsl:variable>

                <!-- Open a private config -->
                <xsl:variable name="rpc-configure-private">
                    <rpc>
                        <open-configuration>
                            <private/>
                        </open-configuration>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>

                <!-- Update all the interfaces tracking the probe -->
                <xsl:for-each
                    select="$ifds//apply-macro[
                    data[name='test-name' and value=$test-name] and 
                    data[name='test-owner' and value=$test-owner] and
                    not(data[name = 'hold-down-time'])]">

                    <!-- Extract some info from the macro and the next-hop's metric -->
                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="hold-down-time">
                        <xsl:choose>
                            <xsl:when test="./data[name='hold-down']/value">
                                <xsl:value-of
                                    select="./data[name='hold-down']/value + $current-time"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="300 +  $current-time"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="ifd-name" select="../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <interfaces>
                                        <interface>
                                            <name>
                                                <xsl:value-of select="$ifd-name"/>
                                            </name>
                                            <disable delete="delete"/>
                                            <apply-macro>
                                                <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                </name>
                                                <data>
                                                  <name>hold-down-time</name>
                                                  <value>
                                                  <xsl:value-of select="$hold-down-time"/>
                                                  </value>
                                                </data>
                                            </apply-macro>
                                        </interface>
                                    </interfaces>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('Interface ',
                        $ifd-name,
                        ' enabled by rpm-track script due to rpm-down event'
                       ))"/>

                </xsl:for-each>

                <!-- Commit the config changes -->
                <xsl:variable name="commit-config-rpc">
                    <rpc>
                        <commit-configuration/>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $commit-config-rpc)"/>

            </xsl:when>

            <xsl:otherwise>
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe down event received, but there is nothing to do with the interfaces')"
                />
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>

    <xsl:template name="ifl-up">
        <xsl:param name="test-owner"/>
        <xsl:param name="test-name"/>

        <!-- First, check if we have anything to change
            Make sure none of the interfaces has a hold-down-time configured
        -->

        <xsl:choose>
            <xsl:when
                test="count(
                $ifls//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                not(data[name = 'hold-down-time'])])">

                <!-- Get current time -->
                <xsl:variable name="current-time">
                    <xsl:call-template name="get-current-time"/>
                </xsl:variable>

                <!-- Open a private config -->
                <xsl:variable name="rpc-configure-private">
                    <rpc>
                        <open-configuration>
                            <private/>
                        </open-configuration>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>

                <!-- Update all the interfaces tracking the probe -->
                <xsl:for-each
                    select="$ifls//apply-macro[
                    data[name='test-name' and value=$test-name] and 
                    data[name='test-owner' and value=$test-owner] and
                    not(data[name = 'hold-down-time'])]">

                    <!-- Extract some info from the macro and the next-hop's metric -->
                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="hold-down-time">
                        <xsl:choose>
                            <xsl:when test="./data[name='hold-down']/value">
                                <xsl:value-of
                                    select="./data[name='hold-down']/value + $current-time"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="300 +  $current-time"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="ifd-name" select="../../name"/>
                    <xsl:variable name="unit-number" select="../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <interfaces>
                                        <interface>
                                            <name>
                                                <xsl:value-of select="$ifd-name"/>
                                            </name>
                                            <unit>
                                                <name>
                                                  <xsl:value-of select="$unit-number"/>
                                                </name>
                                                <disable delete="delete"/>
                                                <apply-macro>
                                                  <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                  </name>
                                                  <data>
                                                  <name>hold-down-time</name>
                                                  <value>
                                                  <xsl:value-of select="$hold-down-time"/>
                                                  </value>
                                                  </data>
                                                </apply-macro>
                                            </unit>
                                        </interface>
                                    </interfaces>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('Interface ',
                        $ifd-name,
                        ' unit ',
                        $unit-number,
                        ' enabled by rpm-track script due to rpm-down event'
                        ))"/>

                </xsl:for-each>

                <!-- Commit the config changes -->
                <xsl:variable name="commit-config-rpc">
                    <rpc>
                        <commit-configuration/>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $commit-config-rpc)"/>

            </xsl:when>

            <xsl:otherwise>
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe down event received, but there is nothing to do with the unit')"
                />
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>

    <xsl:template name="metric-up">
        <xsl:param name="test-owner"/>
        <xsl:param name="test-name"/>
        <xsl:variable name="current-time">
            <xsl:call-template name="get-current-time"/>
        </xsl:variable>

        <!-- First, check if we have anything to change
              Make sure routes have a hold-down-configured (otherwise they are up)
              and that the hold-down-time has expired
        -->
        <xsl:choose>

            <!-- Check that we have routes flaged as down with a hold-down timer expired -->
            <xsl:when
                test="count($routes//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                $current-time > data[name = 'hold-down-time']/value ])">

                <!-- Open a private config -->
                <xsl:variable name="rpc-configure-private">
                    <rpc>
                        <open-configuration>
                            <private/>
                        </open-configuration>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>

                <!-- Update all the routes with unqualified next-hops -->
                <xsl:for-each
                    select="$routes/apply-macro[
                    data[name='test-name' and value=$test-name] and 
                    data[name='test-owner' and value=$test-owner] and
                    $current-time > data[name = 'hold-down-time']/value ]">

                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="weight" select="./data[name='weight']/value"/>
                    <xsl:variable name="metric">
                        <xsl:choose>
                            <xsl:when test="../metric - $weight > 0">
                                <xsl:value-of select="../metric - $weight"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of
                                    select="jcs:syslog(9,'rpm-monitor: Error metric not found or too small while trying
                                    to restore a failed route')"/>
                                <xsl:value-of select="1"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="route-name" select="../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <routing-options>
                                        <static>
                                            <route>
                                                <name>
                                                  <xsl:value-of select="$route-name"/>
                                                </name>
                                                <metric>
                                                  <xsl:value-of select="$metric"/>
                                                </metric>
                                                <apply-macro>
                                                  <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                  </name>
                                                  <data delete="delete">
                                                  <name>hold-down-time</name>
                                                  </data>
                                                </apply-macro>
                                            </route>
                                        </static>
                                    </routing-options>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('rpm-monitor: Metric changed for route ',
                        $route-name, ' to ', $metric, ' due to an up event'))"/>

                </xsl:for-each>


                <!-- Update all the routes with qualified next-hops -->
                <xsl:for-each
                    select="$routes/qualified-next-hop/apply-macro[
                    data[name='test-name' and value=$test-name] and 
                    data[name='test-owner' and value=$test-owner] and
                    $current-time > data[name = 'hold-down-time']/value ]">

                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="weight" select="./data[name='weight']/value"/>
                    <xsl:variable name="metric">
                        <xsl:choose>
                            <xsl:when test="../metric - $weight > 0">
                                <xsl:value-of select="../metric - $weight"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of
                                    select="jcs:syslog(9,'rpm-monitor: Error metric not found or too small while trying
                                    to restore a failed route')"/>
                                <xsl:value-of select="1"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="next-hop-name" select="../name"/>
                    <xsl:variable name="route-name" select="../../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <routing-options>
                                        <static>
                                            <route>
                                                <name>
                                                  <xsl:value-of select="$route-name"/>
                                                </name>
                                                <qualified-next-hop>
                                                  <name>
                                                  <xsl:value-of select="$next-hop-name"/>
                                                  </name>
                                                  <metric>
                                                  <xsl:value-of select="$metric"/>
                                                  </metric>
                                                  <apply-macro>
                                                  <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                  </name>
                                                  <data delete="delete">
                                                  <name>hold-down-time</name>
                                                  </data>
                                                  </apply-macro>
                                                </qualified-next-hop>
                                            </route>
                                        </static>
                                    </routing-options>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('rpm-monitor: Metric changed for route ',
                            $route-name, ' next-hop ', $next-hop-name, ' to ',$metric, ' due to an up event'))"/>

                </xsl:for-each>

                <!-- Commit the config changes -->
                <xsl:variable name="commit-config-rpc">
                    <rpc>
                        <commit-configuration/>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $commit-config-rpc)"/>

            </xsl:when>

            <!-- We have routes flagged as down, but the hold-down time has not yet expired -->
            <xsl:when
                test="count($routes//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                data[name = 'hold-down-time']/value > $current-time ])">
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe up flagged, but the hold-down-time has not yet expired')"
                />
            </xsl:when>

            <xsl:otherwise>
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe up flagged, but there is nothing to do with the routes')"
                />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="ifd-down">
        <xsl:param name="test-owner"/>
        <xsl:param name="test-name"/>
        <xsl:variable name="current-time">
            <xsl:call-template name="get-current-time"/>
        </xsl:variable>

        <!-- First, check if we have anything to change
            Make sure routes have a hold-down-configured (otherwise they are up)
            and that the hold-down-time has expired
        -->
        <xsl:choose>

            <!-- Check that we have routes flaged as down with a hold-down timer expired -->
            <xsl:when
                test="count($ifds//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                $current-time > data[name = 'hold-down-time']/value ])">

                <!-- Open a private config -->
                <xsl:variable name="rpc-configure-private">
                    <rpc>
                        <open-configuration>
                            <private/>
                        </open-configuration>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>

                <!-- Update all the interfaces tracking the rpm probes -->
                <xsl:for-each
                    select="$ifds//apply-macro[
                    data[name='test-name' and value=$test-name] and 
                    data[name='test-owner' and value=$test-owner] and
                    $current-time > data[name = 'hold-down-time']/value ]">

                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="ifd-name" select="../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <interfaces>
                                        <interface>
                                            <name>
                                                <xsl:value-of select="$ifd-name"/>
                                            </name>
                                            <apply-macro>
                                                <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                </name>
                                                <data delete="delete">
                                                  <name>hold-down-time</name>
                                                </data>
                                            </apply-macro>
                                            <disable/>
                                        </interface>
                                    </interfaces>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('rpm-monitor: Interface ',
                        $ifd-name, ' disabled by rpm-monitor due to an RPM up event'))"/>

                </xsl:for-each>

                <!-- Commit the config changes -->
                <xsl:variable name="commit-config-rpc">
                    <rpc>
                        <commit-configuration/>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $commit-config-rpc)"/>

            </xsl:when>

            <!-- We have routes flagged as down, but the hold-down time has not yet expired -->
            <xsl:when
                test="count($ifds//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                data[name = 'hold-down-time']/value > $current-time ])">
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe up flagged, but the hold-down-time of the interface has not yet expired')"
                />
            </xsl:when>

            <xsl:otherwise>
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe up flagged, but there is nothing to do with the interfaces')"
                />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="ifl-down">
        <xsl:param name="test-owner"/>
        <xsl:param name="test-name"/>
        <xsl:variable name="current-time">
            <xsl:call-template name="get-current-time"/>
        </xsl:variable>

        <!-- First, check if we have anything to change
            Make sure routes have a hold-down-configured (otherwise they are up)
            and that the hold-down-time has expired
        -->
        <xsl:choose>

            <!-- Check that we have routes flaged as down with a hold-down timer expired -->
            <xsl:when
                test="count($ifls//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                $current-time > data[name = 'hold-down-time']/value ])">

                <!-- Open a private config -->
                <xsl:variable name="rpc-configure-private">
                    <rpc>
                        <open-configuration>
                            <private/>
                        </open-configuration>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>

                <!-- Update all the interfaces tracking the rpm probes -->
                <xsl:for-each
                    select="$ifls//apply-macro[
                    data[name='test-name' and value=$test-name] and 
                    data[name='test-owner' and value=$test-owner] and
                    $current-time > data[name = 'hold-down-time']/value ]">

                    <xsl:variable name="macro-name" select="./name"/>
                    <xsl:variable name="ifd-name" select="../../name"/>
                    <xsl:variable name="unit-number" select="../name"/>

                    <!-- Create the update RPC changing the metric -->
                    <xsl:variable name="config-update-rpc">
                        <rpc>
                            <load-configuration>
                                <configuration>
                                    <interfaces>
                                        <interface>
                                            <name>
                                                <xsl:value-of select="$ifd-name"/>
                                            </name>
                                            <unit>
                                                <name>
                                                  <xsl:value-of select="$unit-number"/>
                                                </name>
                                                <apply-macro>
                                                  <name>
                                                  <xsl:value-of select="$macro-name"/>
                                                  </name>
                                                  <data delete="delete">
                                                  <name>hold-down-time</name>
                                                  </data>
                                                </apply-macro>
                                                <disable/>
                                            </unit>
                                        </interface>
                                    </interfaces>
                                </configuration>
                            </load-configuration>
                        </rpc>
                    </xsl:variable>

                    <!-- Load the updated config -->
                    <xsl:value-of select="jcs:execute($connection, $config-update-rpc)"/>

                    <!-- Log the change -->
                    <xsl:value-of
                        select="jcs:syslog(12,concat('rpm-monitor: Interface ',
                        $ifd-name, 'unit ', $unit-number, ' disabled by rpm-monitor due to an RPM up event'))"/>

                </xsl:for-each>

                <!-- Commit the config changes -->
                <xsl:variable name="commit-config-rpc">
                    <rpc>
                        <commit-configuration/>
                    </rpc>
                </xsl:variable>
                <xsl:value-of select="jcs:execute($connection, $commit-config-rpc)"/>

            </xsl:when>

            <!-- We have ifls flagged as down, but the hold-down time has not yet expired -->
            <xsl:when
                test="count($ifls//apply-macro[
                data[name='test-name' and value=$test-name] and 
                data[name='test-owner' and value=$test-owner] and
                data[name = 'hold-down-time']/value > $current-time ])">
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe up flagged, but the hold-down-time of the logical interface has not yet expired')"
                />
            </xsl:when>

            <xsl:otherwise>
                <xsl:value-of
                    select="jcs:syslog(12,'rpm-monitor: RPM probe up flagged, but there is nothing to do with the logical interfaces')"
                />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <xsl:template match="/">
        <op-script-results>

            <output>
                <xsl:copy-of select="$configuration"/>
            </output>

            <xsl:value-of
                select="jcs:syslog(12,concat('rpm-monitor: Triggered by ',$event-name, ' test ', $test-name, ' owner ', $test-owner))"/>

            <xsl:choose>
                <xsl:when
                    test="($event-name = 'ping_test_down' 
                        or $event-name = 'ping_test_updown'
                        or $event-name = 'ping_threshold_down') 
                        and ($routes or $ifds or $ifls)">
                    <!-- Modify the metric of the routes monitoring this test -->
                    <xsl:call-template name="metric-down">
                        <xsl:with-param name="test-name" select="$test-name"/>
                        <xsl:with-param name="test-owner" select="$test-owner"/>
                    </xsl:call-template>
                    <xsl:call-template name="ifd-up">
                        <xsl:with-param name="test-name" select="$test-name"/>
                        <xsl:with-param name="test-owner" select="$test-owner"/>
                    </xsl:call-template>
                    <xsl:call-template name="ifl-up">
                        <xsl:with-param name="test-name" select="$test-name"/>
                        <xsl:with-param name="test-owner" select="$test-owner"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when
                    test="$event-name = 'ping_test_up' 
                    or $event-name = 'ping_test_downup' 
                    and ($routes or $ifds or $ifls)">
                    <xsl:call-template name="metric-up">
                        <xsl:with-param name="test-name" select="$test-name"/>
                        <xsl:with-param name="test-owner" select="$test-owner"/>
                    </xsl:call-template>
                    <xsl:call-template name="ifd-down">
                        <xsl:with-param name="test-name" select="$test-name"/>
                        <xsl:with-param name="test-owner" select="$test-owner"/>
                    </xsl:call-template>
                    <xsl:call-template name="ifl-down">
                        <xsl:with-param name="test-name" select="$test-name"/>
                        <xsl:with-param name="test-owner" select="$test-owner"/>
                    </xsl:call-template>
                </xsl:when>
            </xsl:choose>

        </op-script-results>
    </xsl:template>

</xsl:stylesheet>
