<!-- 

fixing newline issue in 10.2 beta

changing \n to &#10;

-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:junos="http://xml.juniper.net/junos/*/junos"
    xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm" xmlns:ext="http://xmlsoft.org/XSLT/namespace"
    xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">
    <xsl:import href="../import/junos.xsl"/>
    
    <xsl:variable name="connection" select="jcs:open()"/>
    
    <xsl:template name="monitor-fpc">
        <xsl:param name="fpc-number"/>
        <xsl:variable name="rpc-security-monitor-fpc">
            <rpc>
                <command>show security monitor fpc <xsl:value-of select="$fpc-number"/> no-forwarding</command>
            </rpc>
        </xsl:variable>
        <xsl:variable name="security-monitor-results" select="jcs:execute($connection,$rpc-security-monitor-fpc)"/>
        <xsl:for-each select="$security-monitor-results//spu-utilization-statistics">
            <xsl:if test="./pic-number[1]">
                <xsl:variable name="fpc-number" select="./fpc-number[1]"/>
                <xsl:variable name="pic-number0" select="./pic-number[1]"/>
                <xsl:variable name="spu-cpu-utilization0" select="./spu-cpu-utilization[1]"/>
                <xsl:variable name="spu-memory-utilization0" select="./spu-memory-utilization[1]"/>
                <xsl:variable name="spu-current-flow-session0" select="./spu-current-flow-session[1]"/>
                <xsl:variable name="spu-max-flow-session0" select="./spu-max-flow-session[1]"/>
                <xsl:variable name="spu-current-cp-session0" select="./spu-current-cp-session[1]"/>
                <xsl:variable name="spu-max-cp-session0" select="./spu-max-cp-session[1]"/>
                <output>
                    <xsl:value-of select="jcs:printf('%3d %3d %3d %3d %14d %14d %14d %14d&#10;', $fpc-number, $pic-number0, $spu-cpu-utilization0, $spu-memory-utilization0, $spu-current-flow-session0, $spu-max-flow-session0, $spu-current-cp-session0, $spu-max-cp-session0)"/>
                </output>
            </xsl:if>
            <xsl:if test="./pic-number[2]">
                <xsl:variable name="fpc-number" select="./fpc-number[1]"/>
                <xsl:variable name="pic-number1" select="./pic-number[2]"/>
                <xsl:variable name="spu-cpu-utilization1" select="./spu-cpu-utilization[2]"/>
                <xsl:variable name="spu-memory-utilization1" select="./spu-memory-utilization[2]"/>
                <xsl:variable name="spu-current-flow-session1" select="./spu-current-flow-session[2]"/>
                <xsl:variable name="spu-max-flow-session1" select="./spu-max-flow-session[2]"/>
                <xsl:variable name="spu-current-cp-session1" select="./spu-current-cp-session[2]"/>
                <xsl:variable name="spu-max-cp-session1" select="./spu-max-cp-session[2]"/>
                <output>
                    <xsl:value-of select="jcs:printf('%3d %3d %3d %3d %14d %14d %14d %14d&#10;', $fpc-number, $pic-number1, $spu-cpu-utilization1, $spu-memory-utilization1, $spu-current-flow-session1, $spu-max-flow-session1, $spu-current-cp-session1, $spu-max-cp-session1)"/>
                </output>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="get-srx-spcs">
        <xsl:variable name="rpc-show-chassis-hardware">
            <rpc>
                <command>show chassis hardware no-forwarding</command>
            </rpc>
        </xsl:variable>
        <xsl:variable name="show-chassis-hardware-results" select="jcs:execute($connection,$rpc-show-chassis-hardware)"/>
        <output>
            <xsl:value-of select="jcs:printf('%4s&#10;', 'SPUs')"/>
            <xsl:value-of select="jcs:printf('%-3s %-3jcs %-3jcs %-3jcs %14s %14s %14s %14s&#10;', 'FPC', 'PIC', 'CPU', 'Mem', 'Flow Sess Cur', 'Flow Ses Max', 'CP Ses Cur', 'CP Ses Max')"/>
            <xsl:value-of select="jcs:printf('%-80j1s&#10;', '--------------------------------------------------------------------------------')"/>
        </output>
        <xsl:variable name="fpc-regex">(FPC )([0-9]+)</xsl:variable>
        
        <xsl:for-each select="$show-chassis-hardware-results//chassis/chassis-module">
            <xsl:choose>
                <xsl:when test="contains(./description,'SPC')">
                    <xsl:variable name="fpc-number" select="jcs:regex($fpc-regex,./name)"/>
                    <xsl:call-template name="monitor-fpc"><xsl:with-param name="fpc-number" select="$fpc-number[3]"/></xsl:call-template>
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template name="get-srx-re">
        <xsl:variable name="rpc-show-chassis-routing-engine">
            <rpc>
                <command>show chassis routing-engine no-forwarding</command>
            </rpc>
        </xsl:variable>
        <xsl:variable name="show-chassis-routing-engine-results" select="jcs:execute($connection,$rpc-show-chassis-routing-engine)"/>
        <output>
            <xsl:value-of select="jcs:printf('%12s&#10;', 'Route Engine')"/>
            <xsl:value-of select="jcs:printf('%-4s %-7jcs %-7jcs %-7jcs %8s %8s %8s %9s %8s&#10;', 'Slot', 'Mem Size', 'Mem Used', 'CPU Avg', 'CPU User', 'CPU Bkgd', 'CPU Krnl', 'CPU Intpt', 'CPU Idle')"/>
            <xsl:value-of select="jcs:printf('%-80j1s&#10;', '--------------------------------------------------------------------------------')"/>
            <xsl:for-each select="$show-chassis-routing-engine-results//route-engine">
                <xsl:value-of select="jcs:printf('%4d %7d %9d %7d %8d %8d %8d %9d %8d&#10;', ./slot,
                                                                                          ./memory-dram-size,
                                                                                          ./memory-buffer-utilization,
                                                                                          (./cpu-user + ./cpu-background + ./cpu-system + ./cpu-interrupt ),
                                                                                          ./cpu-user,
                                                                                          ./cpu-background,
                                                                                          ./cpu-system,
                                                                                          ./cpu-interrupt,
                                                                                          ./cpu-idle)"/>
            </xsl:for-each>
        </output>
    </xsl:template>
    
    
    <xsl:template match="/">
        <op-script-results>
            <xsl:call-template name="get-srx-re"/>
            <xsl:call-template name="get-srx-spcs"/>
        </op-script-results>    
    </xsl:template>
    
</xsl:stylesheet>
