<!-- 
Copyright Juniper Networks 2008
All rights reserved and owned by Juniper Networks

Authors: Rob Cameron (robc@juniper.net) and Patricio Giecco (pgiecco@juniper.net)

-->

<!-- 
	Things to do:
	Add in more logging
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:junos="http://xml.juniper.net/junos/*/junos"
	xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm" xmlns:ext="http://xmlsoft.org/XSLT/namespace"
	xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">
	<xsl:import href="../import/junos.xsl"/>	
	
	<!-- srx-ha-lib.xsl file start -->
	
	  <!-- use a global connection for all rpc connections -->
    <xsl:variable name="connection" select="jcs:open()"/>
    
    <!-- Pull the chassis cluster status and use throughout the script -->
    <xsl:variable name="get-cluster-status">
        <rpc>
            <command>show chassis cluster status</command>
        </rpc>
    </xsl:variable>
    <xsl:variable name="cluster-status-results" select="jcs:execute($connection, $get-cluster-status)"/>
    
    <xsl:variable name="chassis-cluster-rg-rpc">
        <rpc>
            <get-configuration>
                <configuration>
                    <chassis>
                        <cluster/>
                    </chassis>
                </configuration>
            </get-configuration>
        </rpc>
    </xsl:variable>
    <!-- Pull the redundancy group information information out of the configuration, used throughout made global-->
    <xsl:variable name="chassis-cluster-config" select="jcs:execute($connection, $chassis-cluster-rg-rpc)"/>
    
    <!-- use this as a global to determine the interface ownership by chassis model -->
    <xsl:variable name="product-model"><xsl:call-template name="get-product-model"/></xsl:variable>
    
    <xsl:variable name="get-interface-ownership">
        <xsl:choose>
            <xsl:when test="$product-model = 'srx5600'">
                <xsl:variable name="node0-max-interface" select="5"/>
                <xsl:variable name="node0-min-interface" select="0"/>
                <xsl:variable name="node1-max-interface" select="11"/>
                <xsl:variable name="node1-min-interface" select="6"/>
                <max-interface-number>
                    <node0-max><xsl:value-of select="$node0-max-interface"/></node0-max>
                    <node0-min><xsl:value-of select="$node0-min-interface"/></node0-min>
                    <node1-max><xsl:value-of select="$node1-max-interface"/></node1-max>
                    <node1-min><xsl:value-of select="$node1-min-interface"/></node1-min>
                </max-interface-number>
            </xsl:when>
            <xsl:when test="$product-model = 'srx5800'">
                <xsl:variable name="node0-max-interface" select="11"/>
                <xsl:variable name="node0-min-interface" select="0"/>
                <xsl:variable name="node1-max-interface" select="23"/>
                <xsl:variable name="node1-min-interface" select="12"/>
                <max-interface-number>
                    <node0-max><xsl:value-of select="$node0-max-interface"/></node0-max>
                    <node0-min><xsl:value-of select="$node0-min-interface"/></node0-min>
                    <node1-max><xsl:value-of select="$node1-max-interface"/></node1-max>
                    <node1-min><xsl:value-of select="$node1-min-interface"/></node1-min>
                </max-interface-number>
            </xsl:when>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="chassis-interface-ownership" select="ext:node-set($get-interface-ownership)"/>
    
    <!-- end global section -->
       
    <!-- start template section -->
    <!-- Determine which interface are monitored -->
    <xsl:template name="get-monitor-interface-current-weight">
        <xsl:param name="redundancy-group"/>
        <xsl:variable name="results-get-monitor-interface-weight" select="jcs:execute($connection, $chassis-cluster-rg-rpc)"/>
        <xsl:variable name="interface-monitor-weight" select="$results-get-monitor-interface-weight/chassis/cluster/apply-macro[name='failover-interface-monitor']/data[name='$redundancy-group']/value"/>
        <xsl:choose>
            <xsl:when test="boolean($interface-monitor-weight)">
                <text><xsl:value-of select="$interface-monitor-weight"/></text>
            </xsl:when>
            <xsl:otherwise>
                <text>0</text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Determine current time -->
    <xsl:template name="get-current-time">
        <xsl:variable name="rpc-get-current-time">
            <rpc>
                <get-system-uptime-information/>
            </rpc>
        </xsl:variable>
        <xsl:variable name="results-get-current-time" select="jcs:execute($connection, $rpc-get-current-time)"/>
        <xsl:choose>
            <xsl:when test="$results-get-current-time/multi-routing-engine-item">
                <xsl:variable name="current-time" select="$results-get-current-time/multi-routing-engine-item[1]/system-uptime-information/current-time/date-time/@junos:seconds"/>
                <text><xsl:value-of select="$current-time"/></text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="current-time" select="$results-get-current-time/current-time/date-time/@junos:seconds"/>
                <text><xsl:value-of select="$current-time"/></text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Check if RG0 is ready to failover -->
    <xsl:template name="check-RG0-failback">
        <xsl:variable name="rg0-last-failover"><xsl:call-template name="get-rg0-last-failover"/></xsl:variable>
        <xsl:variable name="current-time"><xsl:call-template name="get-current-time"></xsl:call-template></xsl:variable>
        <xsl:variable name="local-node"><xsl:call-template name="get-local-node"></xsl:call-template></xsl:variable>
        <xsl:variable name="rg0-master">
            <xsl:call-template name="get-master">
                <xsl:with-param name="redundancy-group">0</xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rg1-master">
            <xsl:call-template name="get-master">
                <xsl:with-param name="redundancy-group">1</xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rg0-primary-node">
            <!-- Check who is the primary and whether the cluster has not been failed over already -->
            <xsl:choose>
                <xsl:when test="$cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[1]='secondary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[2]='primary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/failover-mode[2]='no'">
                    <!-- failover RG to node0 -->
                    <text>0</text>
                </xsl:when>
                <xsl:when  test="$cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[1]='primary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[2]='secondary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/failover-mode[1]='no'">
                    <!-- failover RG to node1 -->
                    <text>1</text>
                </xsl:when>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="$local-node = $rg0-master">
            <xsl:if test="$rg0-master != $rg1-master">
                <xsl:if test="($current-time) &gt; ($rg0-last-failover + 300)">
                    <xsl:call-template name="request-rg-failover">
                        <xsl:with-param name="node" select="$rg0-primary-node"/>
                        <xsl:with-param name="redundancy-group">0</xsl:with-param>
                    </xsl:call-template>
                </xsl:if>
            </xsl:if>
        </xsl:if>
    </xsl:template>
    
    <!-- Check to to see the manual failover flag needs to be reset, if it does then reset it for all of the rgs
           
    -->
    <xsl:template name="check-and-reset-manual-failover-flag">
        <xsl:if test="boolean($chassis-cluster-config/chassis/cluster/apply-macro[name='monitoring-options']/data[name='clear-failover'])">
            <xsl:variable name="rpc-check-manual-failover-flag">
                <rpc>
                    <command>show chassis cluster status</command>
                </rpc>
            </xsl:variable>
            <xsl:variable name="result-check-manual-failover-flag" select="jcs:execute($connection, $rpc-check-manual-failover-flag)"/>
            <xsl:variable name="result-check-manual-failover-flag-node-set" select="ext:node-set($result-check-manual-failover-flag)"/>
            
            <xsl:for-each select="$result-check-manual-failover-flag//redundancy-group">
                <xsl:if test="./device-stats/failover-mode[1] = 'yes'">
                    <xsl:call-template name="reset-failover-flag">
                        <xsl:with-param name="redundancy-group" select="./redundancy-group-id[1]"/>
                    </xsl:call-template>
                </xsl:if>
            </xsl:for-each>            
        </xsl:if>
    </xsl:template>
    
    
    <!-- Determine the minute in the configuration -->
    <xsl:template name="get-rg0-last-failover">
        <xsl:variable name="results-get-rg0-last-failover" select="jcs:execute($connection, $chassis-cluster-rg-rpc)"/>
        <xsl:variable name="last-rg0-failover-time" select="$results-get-rg0-last-failover/chassis/cluster/apply-macro[name='failover-monitoring']/data[name='last-failover']/value"/>
        <xsl:choose>
            <xsl:when test="boolean($last-rg0-failover-time)">
                <text><xsl:value-of select="$last-rg0-failover-time"/></text>
            </xsl:when>
            <xsl:otherwise>
                <text>0</text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Insert the last minute into the configuration -->
    <xsl:template name="set-rg0-last-failover">
        <xsl:variable name="current-time"><xsl:call-template name="get-current-time"/></xsl:variable>
        <!--<xsl:value-of select="jcs:output(concat('Setting last failover time for RG0 to ', $current-time))"/>-->
        <xsl:variable name="rpc-configure-private">
            <rpc> 
                <open-configuration> 
                    <private/> 
                </open-configuration> 
            </rpc> 
        </xsl:variable>
        <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>
        <xsl:variable name="rpc-set-rg0-last-failover">
            <rpc> 
                <load-configuration>
                    <configuration>
                        <chassis>
                            <cluster>
                                <apply-macro>
                                    <name>failover-monitoring</name>
                                    <data>
                                        <name>last-failover</name>
                                        <value><xsl:value-of select="$current-time"/></value>
                                    </data>
                                </apply-macro>
                            </cluster>
                        </chassis>
                    </configuration>
                </load-configuration> 
            </rpc>
        </xsl:variable>
        <xsl:value-of select="jcs:execute($connection, $rpc-set-rg0-last-failover)"/>
        <xsl:variable name="commit">
            <rpc> 
                <commit-configuration/> 
            </rpc> 
        </xsl:variable>
        <xsl:value-of select="jcs:execute($connection, $commit)"/>
    </xsl:template>
    
    <!-- Insert the last minute into the configuration -->
    <xsl:template name="set-track-interface-last-weight">
        <xsl:param name="weight" select="0"/>
        <xsl:param name="redundancy-group"/>
        <xsl:variable name="current-weight"><xsl:call-template name="get-monitor-interface-current-weight"><xsl:with-param name="redundancy-group" select="$redundancy-group"/></xsl:call-template></xsl:variable>
        <xsl:variable name="total-weight" select="$weight + $current-weight"/>
        <!--<xsl:value-of select="jcs:output(concat('Setting track interface weight to ', $weight, ' for RG ', $redundancy-group))"/>-->
        <!--<xsl:value-of select="jcs:output(concat('Setting last failover time for RG0 to ', $current-time))"/>-->
        <xsl:variable name="rpc-configure-private">
            <rpc> 
                <open-configuration> 
                    <private/> 
                </open-configuration> 
            </rpc> 
        </xsl:variable>
        <xsl:value-of select="jcs:execute($connection, $rpc-configure-private)"/>
        <xsl:variable name="rpc-set-track-interface-last-weight">
            <rpc> 
                <load-configuration>
                    <configuration>
                        <chassis>
                            <cluster>
                                <apply-macro>
                                    <name>failover-interface-monitor</name>
                                    <data>
                                        <name><xsl:value-of select="$redundancy-group"/></name>
                                        <value><xsl:value-of select="$total-weight"/></value>
                                    </data>
                                </apply-macro>
                            </cluster>
                        </chassis>
                    </configuration>
                </load-configuration> 
            </rpc>
        </xsl:variable>
        <xsl:value-of select="jcs:execute($connection, $rpc-set-track-interface-last-weight)"/>
        <xsl:variable name="commit">
            <rpc> 
                <commit-configuration/> 
            </rpc> 
        </xsl:variable>
        <xsl:value-of select="jcs:execute($connection, $commit)"/>
    </xsl:template>
    
    <!-- abstract the actual failover command outside of request failover -->
    <xsl:template name="request-rg-failover">
        <xsl:param name="node"/>
        <xsl:param name="redundancy-group"/>
        
        <xsl:if test="$redundancy-group != ''">
            <!-- rpc command for failover -->
            <xsl:variable name="rpc-failover">
                <rpc>
                    <command>request chassis cluster failover node <xsl:value-of select="$node"/> redundancy-group <xsl:value-of select="$redundancy-group"/></command>
                </rpc>
            </xsl:variable>
            <xsl:value-of select="jcs:execute($connection, $rpc-failover)"/>
        </xsl:if>
        <!-- added to allow command take effect -->
        <xsl:value-of select="jcs:sleep(0,500)"/>
    </xsl:template>
    
    <!--  
        request-failover :: Chassis failover
        This template performs an rg failover of the requested group
        
        @param redundancy-group specifies the redundancy group to failover, defaults to 1
        @param reset-flag specifies the manual failover flag should be cleared, defaults to false
        @param fullfailover-flag specifies if both redundancy groups should be failed over, defaults to faulse
        @param rg0-failover-check specifies if the failover time should be checked, defaults to true
    -->
    <xsl:template name="request-failover">
        <!-- Define which RG to failover -->
        <xsl:param name="redundancy-group">1</xsl:param><!-- force the selection of a parameter disable the default -->
        <xsl:param name="reset-flag"><xsl:value-of select="false()"/></xsl:param>
        <!-- Chosing this forces both RGs to failover -->
        <xsl:param name="fullfailover-flag">0</xsl:param>
        <!-- Verify if its safe to failover RG0 -->
        <xsl:param name="rg0-failover-check">1</xsl:param>
        
        <!-- determite the other RG that would need to failover in a full failover -->
        <xsl:variable name="other-redundancy-group">
            <xsl:choose>
                <xsl:when test="$redundancy-group = 0"><text>1</text></xsl:when>
                <xsl:when test="$redundancy-group = 1"><text>0</text></xsl:when>
            </xsl:choose>
        </xsl:variable>
        
        <!-- used to determine if we should fully failover -->
        <xsl:variable name="rg0-master">
            <xsl:call-template name="get-master">
                <xsl:with-param name="redundancy-group">0</xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rg1-master">
            <xsl:call-template name="get-master">
                <xsl:with-param name="redundancy-group">1</xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rg0-last-failover"><xsl:call-template name="get-rg0-last-failover"/></xsl:variable>
        <xsl:variable name="current-time"><xsl:call-template name="get-current-time"></xsl:call-template></xsl:variable>
        
        <!--<xsl:variable name="local-node">
            <xsl:call-template name="get-local-node"/>
        </xsl:variable>-->
        
        <xsl:variable name="rg-primary-node">
            <!-- Check who is the primary and whether the cluster has not been failed over already -->
            <xsl:choose>
                <xsl:when test="$cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[1]='secondary' 
                    and $cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[2]='primary' 
                    and $cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/failover-mode[2]='no'">
                    <!-- failover RG to node0 -->
                    <text>0</text>
                </xsl:when>
                <xsl:when  test="$cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[1]='primary' 
                    and $cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[2]='secondary' 
                    and $cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/failover-mode[1]='no'">
                    <!-- failover RG to node1 -->
                    <text>1</text>
                </xsl:when>
            </xsl:choose>
        </xsl:variable>
        
        <xsl:variable name="rg0-primary-node">
            <!-- Check who is the primary and whether the cluster has not been failed over already -->
            <xsl:choose>
                <xsl:when test="$cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[1]='secondary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[2]='primary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/failover-mode[2]='no'">
                    <!-- failover RG to node0 -->
                    <text>0</text>
                </xsl:when>
                <xsl:when  test="$cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[1]='primary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/redundancy-group-status[2]='secondary' 
                    and $cluster-status-results/redundancy-group[1]/device-stats/failover-mode[1]='no'">
                    <!-- failover RG to node1 -->
                    <text>1</text>
                </xsl:when>
            </xsl:choose>
        </xsl:variable>
        
        <!-- 
            Execute the failover to the other chassis
        -->

        <xsl:choose>
            <!-- check if redundancy group 0 needs to be checked before failover -->
            <xsl:when test="boolean($rg0-failover-check = 1)">
                <xsl:if test="boolean($rg-primary-node != '')">
                    <xsl:choose>
                        <!-- Thus is used when a full failover  -->
                        <xsl:when test="boolean($fullfailover-flag and ((($rg0-master = 'node0') and ($rg1-master = 'node0')) or (($rg0-master = 'node1') and ($rg1-master = 'node1'))))">
                            <xsl:choose>
                                <xsl:when test="($current-time) &gt; ($rg0-last-failover + 300)">
                                    <xsl:variable name="time-diff" select="$current-time - $rg0-last-failover"/>
                                    <xsl:value-of select="jcs:syslog(146,concat($time-diff, ' seconds since last failover of RG0. Failing over RG0 to node', $rg-primary-node))"/>
                                    <!--<xsl:value-of select="jcs:output(concat($time-diff, ' seconds since last failover of RG0. Failing over RG0.'))"/>-->
                                    <!--<xsl:value-of select="jcs:output('Requesting failover for RG1')"/>-->
                                    <xsl:value-of select="jcs:syslog(146,concat('Requesting failover for RG1 to node', $rg-primary-node ))"/>
                                    <xsl:call-template name="request-rg-failover">
                                        <xsl:with-param name="node" select="$rg-primary-node"/>
                                        <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                                    </xsl:call-template>
                                    <!-- full failover matched requesting RG0 failover -->
                                    <xsl:call-template name="request-rg-failover">
                                        <xsl:with-param name="node" select="$rg0-primary-node"/>
                                        <xsl:with-param name="redundancy-group" select="$other-redundancy-group"/>
                                    </xsl:call-template>
                                    <xsl:call-template name="set-rg0-last-failover"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:variable name="time-diff" select="$current-time - $rg0-last-failover"/>
                                    <xsl:value-of select="jcs:syslog(146,concat('Not enough time has passed to failover RG0 only ', $time-diff, ' seconds have passed on node', $rg0-primary-node))"/>
                                    <!--<xsl:value-of select="jcs:output(concat('Not enough time has passed to failover RG0 only ', $time-diff, ' seconds have passed'))"/>-->
                                    <!--<xsl:value-of select="jcs:output('Requesting failover for RG1')"/>-->
                                    <xsl:value-of select="jcs:syslog(146,concat('Requesting failover for RG1 to node', $rg-primary-node ))"/>
                                    <xsl:call-template name="request-rg-failover">
                                        <xsl:with-param name="node" select="$rg-primary-node"/>
                                        <xsl:with-param name="redundancy-group">1</xsl:with-param>
                                    </xsl:call-template>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <!-- This is used when redundancy group 0 specificially was called to failover -->
                        <xsl:when test="$redundancy-group = 0">
                            <xsl:choose>
                                <xsl:when test="$current-time &gt; ($rg0-last-failover + 300)">
                                    <xsl:variable name="time-diff" select="$current-time - $rg0-last-failover"/>
                                    <!--<xsl:value-of select="jcs:output(concat($time-diff, ' seconds since last failover of RG0. Failing over RG0.'))"/>-->
                                    <xsl:value-of select="jcs:syslog(146,concat($time-diff, ' seconds since last failover of RG0. Failing over RG0 to node', $rg-primary-node))"/>
                                    <xsl:call-template name="request-rg-failover">
                                        <xsl:with-param name="node" select="$rg-primary-node"/>
                                        <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                                    </xsl:call-template>
                                    <xsl:call-template name="set-rg0-last-failover"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:variable name="time-diff" select="$current-time - $rg0-last-failover"/>
                                    <!--<xsl:value-of select="jcs:output(concat('Not enough time has passed to failover RG0 over ', $time-diff, ' seconds have passed'))"/>-->
                                    <xsl:value-of select="jcs:syslog(146,concat('Not enough time has passed to failover RG0 only ', $time-diff, ' seconds have passed on node', $rg0-primary-node))"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <!-- This is used when redundancy group 1 specificially was called to failover -->
                        <xsl:when test="$redundancy-group = 1">
                            <xsl:value-of select="jcs:syslog(146,concat('Requesting failover for RG1 to node', $rg-primary-node ))"/>
                            <xsl:call-template name="request-rg-failover">
                                <xsl:with-param name="node" select="$rg-primary-node"/>
                                <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                            </xsl:call-template>
                        </xsl:when>
                    </xsl:choose>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:if test="boolean($rpc-failover != '')">
                    <xsl:choose>
                        <!-- Thus is used when a full failover  -->
                        <xsl:when test="boolean($fullfailover-flag)">
                            <!--<xsl:value-of select="jcs:output('Requesting full failover for both RG0 and RG1')"/>-->
                            <xsl:value-of select="jcs:syslog(146,concat('Requesting failover for RG0 and RG1 to node', $rg-primary-node ))"/>
                            <xsl:call-template name="request-rg-failover">
                                <xsl:with-param name="node" select="$rg-primary-node"/>
                                <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                            </xsl:call-template>
                            <!-- full failover matched requesting RG0 failover -->
                            <xsl:call-template name="request-rg-failover">
                                <xsl:with-param name="node" select="$rg0-primary-node"/>
                                <xsl:with-param name="redundancy-group" select="$other-redundancy-group"/>
                            </xsl:call-template>
                            <xsl:call-template name="set-rg0-last-failover"/>
                        </xsl:when>
                        <!-- This is used when redundancy group 0 specificially was called to failover -->
                        <xsl:when test="$redundancy-group = 0">
                            <!--<xsl:value-of select="jcs:output('Requesting failover for RG0')"/>-->
                            <xsl:value-of select="jcs:syslog(146,concat('Requesting failover for RG0 to node', $rg0-primary-node ))"/>
                            <xsl:call-template name="request-rg-failover">
                                <xsl:with-param name="node" select="$rg-primary-node"/>
                                <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                            </xsl:call-template>
                            <xsl:call-template name="set-rg0-last-failover"/>
                        </xsl:when>
                        <!-- This is used when redundancy group 1 specificially was called to failover -->
                        <xsl:when test="$redundancy-group = 1">
                            <!--<xsl:value-of select="jcs:output('Requesting failover for RG1')"/>-->
                            <xsl:value-of select="jcs:syslog(146,concat('Requesting failover for RG1 to node', $rg-primary-node ))"/>
                            <xsl:call-template name="request-rg-failover">
                                <xsl:with-param name="node" select="$rg-primary-node"/>
                                <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                            </xsl:call-template>
                        </xsl:when>
                    </xsl:choose>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
        
        <!-- 
            Clear the failover status if requested
        -->
        <xsl:if test="boolean($reset-flag)">
            <xsl:choose>
                <xsl:when test="boolean($fullfailover-flag)">
                    <xsl:call-template name="reset-failover-flag">
                        <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                    </xsl:call-template>           
                    <xsl:call-template name="reset-failover-flag">
                        <xsl:with-param name="redundancy-group" select="$other-redundancy-group"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="reset-failover-flag">
                        <xsl:with-param name="redundancy-group" select="$redundancy-group"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    
    </xsl:template>
    
    <!--	
        reset-failover-flag :: Get Master
        This template is used to reset the failover flag 
        
        @param redundancy-group specifies the redundancy group to failover, 1 is the default
    -->
    <xsl:template name="reset-failover-flag">
        <xsl:param name="redundancy-group">1</xsl:param>
        <xsl:variable name="rpc-clear-failover-flag">
            <rpc>
                <command>request chassis cluster failover reset redundancy-group <xsl:value-of select="$redundancy-group"/></command>
            </rpc>
        </xsl:variable>
        <!--<xsl:value-of
        select="jcs:output(concat('Reseting manual failover for redundancy group ', $redundancy-group))"/>-->
        <xsl:value-of select="jcs:syslog(146,concat('Reseting manual failover for redundancy group ', $redundancy-group))"/>
        <xsl:value-of select="jcs:execute($connection, $rpc-clear-failover-flag)"/>	
        <xsl:value-of select="jcs:sleep(0,500)"/>
    </xsl:template>
    
    <!--	
        get-master :: Get Master
        This template is used to determine device is master of a redundancy group (RG0 by default). 
        
        @param redundancy-group specifies the redundancy group to check master default is 0
    -->
    <xsl:template name="get-master">
        <xsl:param name="redundancy-group">0</xsl:param> 
        <!-- Determine the master of the chassis cluster-->
        <xsl:variable name="rg-node0-priority" select="$cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[1]"/>
        <xsl:variable name="rg-node1-priority" select="$cluster-status-results/redundancy-group[$redundancy-group+1]/device-stats/redundancy-group-status[2]"/>
        <xsl:choose>
            <xsl:when test="not($rg-node0-priority) or not($rg-node1-priority)">
                <!-- One Device is not communicating to the other the test will not run-->
                <!--<xsl:value-of
                    select="jcs:output('Only one cluster member has been found. Check connectivity to the other cluster member.')"
                />-->
            </xsl:when>
            <xsl:when test="$rg-node0-priority = 'primary' and $rg-node1-priority != 'primary'">
                <!-- Node0 is the primary returning Node0-->
                <!--<xsl:value-of select="jcs:output(concat('Node0 is the master for redundancy group: ', $redundancy-group))"/>-->
                <text>node0</text>
            </xsl:when>
            <xsl:when test="$rg-node0-priority != 'primary' and $rg-node1-priority= 'primary'">
                <!-- Node1 is the primary returning Node1-->
                <!--<xsl:value-of select="jcs:output(concat('Node1 is the master for redundancy group: ', $redundancy-group))"/>-->
                <text>node1</text>
            </xsl:when>
            <xsl:otherwise>
                <!-- priorities are the same -->
                <!--<xsl:value-of select="jcs:output('An unexpected result has occured while checking the node status.')" />-->
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Return the local node value -->
    <xsl:template name="get-local-node">
        <xsl:variable name="get-local-RE">
            <rpc>
                <command>show chassis routing-engine node local</command>
            </rpc>
        </xsl:variable>
        <!-- Get the local RE node-->
        <xsl:variable name="local-check-results" select="jcs:execute($connection, $get-local-RE)"/>
        <text><xsl:value-of select="$local-check-results/multi-routing-engine-item/re-name"/></text>
    </xsl:template>
    
    <!--	
        get-manual-failover-flag :: Get manual failover flag
        This template is used to determine if the failover flag is set 
        
        @param redundancy-group specifies the redundancy group to check master default is 0
    -->
    <xsl:template name="get-manual-failover-flag">
        <xsl:param name="redundancy-group"/>
        <xsl:variable name="rpc-check-manual-failover-flag">
            <rpc>
                <command>show chassis cluster status</command>
            </rpc>
        </xsl:variable>
        <xsl:variable name="result-check-manual-failover-flag" select="jcs:execute($connection, $rpc-check-manual-failover-flag)"/>
        <text><xsl:value-of select="$result-check-manual-failover-flag/redundancy-group[$redundancy-group+1]/device-stats/failover-mode[1]"/></text>
    </xsl:template>
    
    <!-- determine and return the current product model -->
    <xsl:template name="get-product-model">
        <xsl:variable name="rpc-product-model">
            <rpc>
                <command>show version</command>
            </rpc>
        </xsl:variable>
        <xsl:variable name="results-product-model" select="jcs:execute($connection, $rpc-product-model)"/>
        <xsl:choose>
            <xsl:when test="$results-product-model/multi-routing-engine-item">
                <text><xsl:value-of select="$results-product-model/multi-routing-engine-item[1]/software-information/product-model"/></text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$results-product-model/software-information/product-model"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- end template section -->
    
	<!-- srx-ha-lib.xsl file end -->
	
	<!-- Global variables -->
	
	<xsl:variable name="local-node-name"><xsl:call-template name="get-local-node"></xsl:call-template></xsl:variable>
	
	<!-- Determine the master for RG0 -->
	<xsl:variable name="master-node-name">
		<xsl:call-template name="get-master">
			<xsl:with-param name="redundancy-group">0</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name="master-RG1">
		<xsl:call-template name="get-master">
			<xsl:with-param name="redundancy-group">1</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name="node-regex">(node)([0-9]+)</xsl:variable>
	<xsl:variable name="local-node-number" select="jcs:regex($node-regex,$local-node-name)"/>
	<xsl:variable name="rg1-node-number" select="jcs:regex($node-regex,$master-RG1)"/>
	
	<!-- 
		ping-server :: Ping server
		This template returns the status of the server by using
		the ping rpc provided by JUNOS
		
		@param host: the IP address or hostname of the system
		@param interval: the time between ping attempts 1 second is the default
		@param count: the total number of ping attempts 3 attepmts is the default
	-->
	<xsl:template name="ping-server">
		<!-- Optimize template to provide defaults for params -->
		<xsl:param name="host"/>
		<xsl:param name="interval"/>
		<xsl:param name="count"/>
		<xsl:param name="wait"/>
		<xsl:param name="routing-instance"/>
		<!-- This variable is the JUNOScript RPC call to execute the ping -->
		<xsl:variable name="rpc-ping">
			<rpc>
				<ping>
					<host><xsl:value-of select="$host"/></host>	
						<xsl:choose>
							<xsl:when test="boolean($interval)">
								<interval><xsl:value-of select="$interval"/></interval>
							</xsl:when>
							<xsl:otherwise>
								<interval>0.1</interval>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:choose>
							<xsl:when test="boolean($count)">
								<count><xsl:value-of select="$count"/></count>
							</xsl:when>
							<xsl:otherwise>
								<count>3</count>
							</xsl:otherwise>
						</xsl:choose>
					<xsl:choose>
						<xsl:when test="boolean($wait)">
							<wait><xsl:value-of select="$wait"/></wait>
						</xsl:when>
						<xsl:otherwise>
							<wait>1</wait>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:if test="boolean($routing-instance)">
						<routing-instance><xsl:value-of select="$routing-instance"/></routing-instance>
					</xsl:if>
					<verbose/>
				</ping>
			</rpc>
		</xsl:variable>
		
		<!-- contains the results from the ping test -->
		<xsl:variable name="ping-results" select="jcs:execute($connection, $rpc-ping)"/>
		<!-- counts the number of successful pings -->
		<xsl:variable name="ping-ok" select="count($ping-results/probe-result/probe-success)"/>
		<!--<xsl:value-of select="jcs:output(concat('SUCCESS COUNT ', $ping-ok))"/>-->
		<xsl:choose>
			<xsl:when test="$ping-ok != 0">ok</xsl:when>
			<xsl:otherwise>nok</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- 
		main :: The starting point and main template for track ip
		This is the main loop for the test. 
		It executes the ping tests and then will failover the cluster if there is an issue.
		
		@param count the specified number of times this loop should run
	-->
	<xsl:template name="main">
		<xsl:param name="count"/>

		<!-- check to see if we need to ruin through the loop again -->
		<xsl:choose>
			<xsl:when test="$count > 0">
			<!-- Log the iteration number -->
			<!--<xsl:value-of select="jcs:output(concat('checking servers ',$count,' iterations to go'))"/>-->
			<!-- This variable stores the list of RGs that need to be failed over -->
			<xsl:variable name="ping-test-results">
				<redundancy-groups>
					<xsl:for-each select="$chassis-cluster-config//cluster/redundancy-group">
						<xsl:variable name="rg" select="./name"/>
						<redundancy-group>
							<name>
								<xsl:value-of select="$rg"/>
							</name>
							<!-- Loop through the apply-macro track-* statements and check each server
							adds a server node to the redundancy group for each host tracked           -->
							<xsl:for-each select="./apply-macro[starts-with(name,'track-')]">
								<xsl:variable name="server-status">
									<!-- Ping with the user specified ping count -->
									<xsl:call-template name="ping-server">
										<xsl:with-param name="host" select="./data[name='server']/value"/>
										<xsl:with-param name="count" select="./data[name='count']/value"/>
										<xsl:with-param name="wait" select="./data[name='wait']/value"/>
										<xsl:with-param name="interval" select="./data[name='interval']/value"/>
										<xsl:with-param name="routing-instance" select="./data[name='routing-instance']/value"/>
									</xsl:call-template>
								</xsl:variable>
								
								<xsl:choose>
									<xsl:when test="$server-status='nok'">
										<server>
											<name>
												<xsl:value-of select="name"/> 
											</name>
											<weight>
												<xsl:value-of select="./data[name='weight']/value"/>
											</weight>
										</server>
									</xsl:when>
									<xsl:when test="$server-status='ok'"> 
										<!-- Do nothing -->
									</xsl:when>
								</xsl:choose>
							</xsl:for-each>
						</redundancy-group>
					</xsl:for-each>
				</redundancy-groups>
			</xsl:variable>				
			<xsl:variable name="rg-ping-results" select="ext:node-set($ping-test-results)"/>
		
			<xsl:choose>
				<!-- determine if weight has exceeded 254 -->
				<xsl:when test="$rg-ping-results/redundancy-groups/redundancy-group[sum(server/weight) &gt; 254]">
					<xsl:value-of select="jcs:output($ping-test-results)"/>
					<!-- failover each redundancy group that exceeds 254 -->
					<xsl:for-each select="$rg-ping-results/redundancy-groups/redundancy-group[sum(server/weight) &gt; 254]">
						<!--<xsl:value-of select="jcs:output(concat('RG:', name ,' needs to be failed over'))"/>-->
						<xsl:call-template name="request-failover">
							<xsl:with-param name="redundancy-group" select="name"/>
							<xsl:with-param name="reset-flag" select="boolean($chassis-cluster-config/chassis/cluster/apply-macro[name='monitoring-options']/data[name='clear-failover'])"/>
							<xsl:with-param name="fullfailover-flag" select="boolean($chassis-cluster-config/chassis/cluster/apply-macro[name='monitoring-options']/data[name='full-failover'])"/>
						</xsl:call-template>
					</xsl:for-each>
				</xsl:when>
				<!-- servers are ok no need to failover -->
				<xsl:otherwise>
					<!--<xsl:value-of select="jcs:output('Tested servers are ok')"/>-->

					<xsl:call-template name="main">
						<xsl:with-param name="count" select="$count - 1"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise>
			<!-- No more interations to run -->
			<!-- determine if RG0 should follow the leader RG1 -->
			<xsl:call-template name="check-RG0-failback"/>
		</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="/">
		<op-script-results>
			<!-- In here we loop through the Track-ip statements and execute pings against them -->
			<xsl:choose>
				<xsl:when test="($local-node-name = $master-node-name) and $cluster-status-results/redundancy-group[2]/device-stats[1]/device-priority[$rg1-node-number[3]+1] != 0">
					<!--<xsl:value-of select="jcs:output('Node is the master of RG0, cheking tracked objects')"/>-->
					<!-- Figure out how long it takes to check all the servers -->
					<xsl:variable name="runtimes">
						<exec-time>
							<xsl:for-each select="$chassis-cluster-config//cluster/redundancy-group">
								<xsl:for-each select="./apply-macro[starts-with(name,'track-')]">
									<time>
										<xsl:variable name="count">
											<xsl:choose>
												<xsl:when test="boolean(./data[name='count']/value)">
												<xsl:value-of select="./data[name='count']/value"/>
												</xsl:when>
												<xsl:otherwise>3</xsl:otherwise>
											</xsl:choose>
										</xsl:variable>
										<xsl:variable name="interval">
											<xsl:choose>
												<xsl:when
												test="boolean(./data[name='interval']/value)">
												<xsl:value-of
												select="./data[name='interval']/value"/>
												</xsl:when>
												<xsl:otherwise>0.1</xsl:otherwise>
											</xsl:choose>
										</xsl:variable>
										<xsl:variable name="wait">
											<xsl:choose>
												<xsl:when test="boolean(./data[name='wait']/value)">
													<xsl:value-of select="./data[name='wait']/value"/>
												</xsl:when>
												<xsl:otherwise>1</xsl:otherwise>
											</xsl:choose>
										</xsl:variable>
										<xsl:value-of select="(($count * $interval) + ($wait - 1)) * 0.668"/>
									</time>
								</xsl:for-each>
							</xsl:for-each>
						</exec-time>
					</xsl:variable>
					<xsl:variable name="total-runtime" select="sum(ext:node-set($runtimes)//time)"/>
					<!--<xsl:value-of
						select="jcs:output(concat('It will take at most ',$total-runtime, ' seconds to run each iteration'))"/>-->
					<xsl:variable name="execute-iterations" select="ceiling(60 div $total-runtime)"/>
					<xsl:call-template name="main">
						<xsl:with-param name="count" select="$execute-iterations"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="$cluster-status-results/redundancy-group[2]/device-stats[1]/device-priority[$rg1-node-number[3]+1] = 0">
					<!--<xsl:value-of
						select="jcs:output('Node is the maser of RG0. RG1 master in priority 0 state. Nothing to do!')"/>-->
				</xsl:when>
				<xsl:otherwise>
					<!--This note is not master so it shouldn't ping as it will fail-->
					<!--<xsl:value-of
						select="jcs:output('Node is not the maser of RG0. Nothing to do!')"/>-->
				</xsl:otherwise>
			</xsl:choose>
		</op-script-results>
	</xsl:template>
</xsl:stylesheet>